# -*- coding: utf-8 -*-
import scrapy, re, json
from datetime import datetime, date
from amandala.items import NoticiasItem

"""
MEDIO:
Amandala, Belice
USO:
// Si se quiere obtener todas las noticias desde las más actuales hasta las más antiguas. //
scrapy crawl noticias --nolog -s filename=noticias.json
-------------------------------------------------------------------------------------------------
// Si se quiere obtener todas las noticias desde las más actuales hasta una fecha específica. //
scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=3 -a day=5
-------------------------------------------------------------------------------------------------
Después será necesario hacer uso del archivo parse_date_files.py para que las noticias contenidas
en noticias.json sean separadas en archivos por fecha.
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

DAT_RE = re.compile(r'\d{4}\/\d{2}\/\d{2}')


class ImportantData(scrapy.Item):
    CONTINUE_SEARCHING = scrapy.Field()
    LAST_LINK = scrapy.Field()
    url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        baseURL = "http://amandala.com.bz/news/category/"

        sectionList = ["headline", "highlights", "general", "features",
                       "sports", "letters", "editorial", "publisher"]
        # sectionList = ["headline"]

        if self.stopDate is None:
            for s in sectionList:
                yield scrapy.Request(url=baseURL + s, callback=self.parse)

        else:
            for s in sectionList:
                info = ImportantData()
                info['CONTINUE_SEARCHING'] = False
                request = scrapy.Request(url=baseURL + s, callback=self.parse_with_stop_date)
                request.meta['item'] = info
                yield request


    def parse(self, response):
        linkList = response.xpath('//div[@id="list_categories"]').css('div.content').css('h3').css('a::attr(href)').extract()

        for link in linkList:
            yield scrapy.Request(url=link, callback=self.parse_item)

        nextPage = response.xpath('//div[@id="entries"]').css('div.old_entries').css('a::attr(href)').extract_first()
        if nextPage is not None:
            yield scrapy.Request(url=nextPage, callback=self.parse)


    def parse_with_stop_date(self, response):
        searchData = response.meta['item']
        CONTINUE_SEARCHING = searchData['CONTINUE_SEARCHING']

        if not CONTINUE_SEARCHING:
            linkList = response.xpath('//div[@id="list_categories"]').css('div.content').css('h3').css('a::attr(href)').extract()

            for link in linkList:
                info = ImportantData()
                info['url'] = response.url
                if link == linkList[-1]: info['LAST_LINK'] = True
                else: info['LAST_LINK'] = False
                reqst = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                reqst.meta['item'] = info
                yield reqst

        else:
            nextPage = response.xpath('//div[@id="entries"]').css('div.old_entries').css('a::attr(href)').extract_first()
            if nextPage is not None:
                searchData['CONTINUE_SEARCHING'] = False
                request = scrapy.Request(url=nextPage, callback=self.parse_with_stop_date)
                request.meta['item'] = searchData
                yield request


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        d = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        item['date'] = datetime.strptime(d, '%Y-%m-%d').isoformat("T")
        item['title'] = remove_tags(response.xpath('//div[@class="active"]/h1/a').extract_first())

        try:
            topic = response.css('div.date').css('span.date').css('a::text').extract()[0]
        except:
            topic = None
        item['topic'] = topic

        for p in response.css('div.content').css('p').extract():
            text += remove_tags(p) + "\n"

        item['text'] = text.strip()

        item['url'] = response.url

        yield item


    def parse_item_with_stop_date(self, response):
        d = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        dt = datetime.strptime(d, '%Y-%m-%d').date()

        if dt >= self.stopDate:
            info = response.meta['item']
            item = NoticiasItem()
            text = ''

            item['date'] = datetime.strptime(d, '%Y-%m-%d').isoformat("T")
            item['title'] = remove_tags(response.xpath('//div[@class="active"]/h1/a').extract_first())

            try:
                topic = response.css('div.date').css('span.date').css('a::text').extract()[0]
            except:
                topic = None
            item['topic'] = topic

            for p in response.css('div.content').css('p').extract():
                text += remove_tags(p) + "\n"

            item['text'] = text.strip()

            item['url'] = response.url

            yield item

            if info['LAST_LINK']:
                info['CONTINUE_SEARCHING'] = True
                request = scrapy.Request(url=info['url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = info
                yield request
